#! /usr/bin/perl

use strict;
use warnings;

my %code = ('AAA', 'K', 'AAC', 'N', 'AAG', 'K', 'AAT', 'N',
            'ACA', 'T', 'ACC', 'T', 'ACG', 'T', 'ACT', 'T',
            'AGA', 'R', 'AGC', 'S', 'AGG', 'R', 'AGT', 'S',
            'ATA', 'I', 'ATC', 'I', 'ATG', 'M', 'ATT', 'I',
            'CAA', 'Q', 'CAC', 'H', 'CAG', 'Q', 'CAT', 'H',
            'CCA', 'P', 'CCC', 'P', 'CCG', 'P', 'CCT', 'P',
            'CGA', 'R', 'CGC', 'R', 'CGG', 'R', 'CGT', 'R',
            'CTA', 'L', 'CTC', 'L', 'CTG', 'L', 'CTT', 'L',
            'GAA', 'E', 'GAC', 'D', 'GAG', 'E', 'GAT', 'D',
            'GCA', 'A', 'GCC', 'A', 'GCG', 'A', 'GCT', 'A',
            'GGA', 'G', 'GGC', 'G', 'GGG', 'G', 'GGT', 'G',
            'GTA', 'V', 'GTC', 'V', 'GTG', 'V', 'GTT', 'V',
            'TAA', '*', 'TAC', 'Y', 'TAG', '*', 'TAT', 'Y',
            'TCA', 'S', 'TCC', 'S', 'TCG', 'S', 'TCT', 'S',
            'TGA', '*', 'TGC', 'C', 'TGG', 'W', 'TGT', 'C',
            'TTA', 'L', 'TTC', 'F', 'TTG', 'L', 'TTT', 'F');

my %rev = ('A', 'T',
           'T', 'A',
	   'C', 'G',
	   'G', 'C',
	   'N', 'N');

main();

sub main {
	
	my $length = @ARGV;

	my $file = "";
	my $outfile = "";

	if ($length < 3){
		print "Call program like this: perl 6FrameTrans_cp.pl -i <sequence-file> -o <output-file>\n";
	}
	else {
		foreach (my $i=0; $i<$length; $i++){
			if ($ARGV[$i] eq "-i"){
				$file = $ARGV[$i+1];
			}
			if ($ARGV[$i] eq "-o"){
				$outfile = $ARGV[$i+1];
			}
		}
	}

	my $gi = "";
	my $nuc_seq = "";

	open(NUC, "$file");

	# read nucleotide sequences
	while(<NUC>){
		chomp($_);
		if ($_ =~ />(.+)/){
			if ($gi ne ""){
				translate($outfile, $gi, $nuc_seq);
			}
			$gi = $1;
			$nuc_seq = "";
		}
		else {
			$nuc_seq = $nuc_seq . $_;
		}
	}
	translate($outfile, $gi, $nuc_seq);
	close(NUC);

}

sub translate{
	

	my ($outfile, $gi, $nuc_seq) = @_;
	
	open(AMI, ">>$outfile");

	# translate nucleotide sequence in all reading frames and find out which one is the right one

	my %aminoseq;
	my %reverse;

	print "$gi \n";
	# three forward reading frames;
	$aminoseq{1} = "";
	$aminoseq{2} = "";
	$aminoseq{3} = "";
	for (my $i = 0; $i < length($nuc_seq) - 3; $i += 3){
		if (uc(substr($nuc_seq, $i, 3)) =~ /[^ACGT]/){
			$aminoseq{1} .= "X";
		}
		else {
			$aminoseq{1} .= $code{uc(substr($nuc_seq, $i, 3))};
		}
		if (uc(substr($nuc_seq, $i+1, 3)) =~ /[^ACGT]/){
			$aminoseq{2} .= "X";
		}
		else {
			$aminoseq{2} .= $code{uc(substr($nuc_seq, $i+1, 3))};
		}
		if (uc(substr($nuc_seq, $i+2, 3)) =~ /[^ACGT]/){
			$aminoseq{3} .= "X";
		}
		else {
			$aminoseq{3} .= $code{uc(substr($nuc_seq, $i+2, 3))};
		}
	}
	print AMI ">1_$gi\n" . $aminoseq{1} . "\n";
	print AMI ">2_$gi\n" . $aminoseq{2} . "\n";
	print AMI ">3_$gi\n" . $aminoseq{3} . "\n";
	# three backward reading frames;
	$reverse{$gi} = "";
	for (my $i = length($nuc_seq) - 1; $i >= 0; $i--){
		if (defined $rev{uc(substr($nuc_seq, $i, 1))}){
			$reverse{$gi} .= $rev{uc(substr($nuc_seq, $i, 1))};
		}
		else {
			$reverse{$gi} .= 'N';
		}
	}
	$aminoseq{4} = "";
	$aminoseq{5} = "";
	$aminoseq{6} = "";
	for (my $i = 0; $i < length($reverse{$gi}) - 3; $i += 3){
		if (uc(substr($reverse{$gi}, $i, 3)) =~ /[^ACGT]/){
			$aminoseq{4} .= "X";
		}
		else {
			$aminoseq{4} .= $code{uc(substr($reverse{$gi}, $i, 3))};
		}
		if (uc(substr($reverse{$gi}, $i+1, 3)) =~ /[^ACGT]/){
			$aminoseq{5} .= "X";
		}
		else {
			$aminoseq{5} .= $code{uc(substr($reverse{$gi}, $i+1, 3))};
		}
		if (uc(substr($reverse{$gi}, $i+2, 3)) =~ /[^ACGT]/){
			$aminoseq{6} .= "X";
		}
		else {
			$aminoseq{6} .= $code{uc(substr($reverse{$gi}, $i+2, 3))};
		}
	}
	print AMI ">4_$gi\n" . $aminoseq{4} . "\n";
	print AMI ">5_$gi\n" . $aminoseq{5} . "\n";
	print AMI ">6_$gi\n" . $aminoseq{6} . "\n";

	close(AMI);

}
