
#Pipeline for the prediction of MADS-box genes in whole genome
#by Lydia Gramzow and Thomas Winterberg
#
#Required files, folders, programs including naming scheme:
#- whole-genome-sequences (nucleotide): >name<_RM.fa
#- whole-genome-sequences (aminoacid): >name<_RM.aa
#- coding-sequences (nucleotide): >name<_cds.fa
#- Hidden Markov Model for MADS domain: MADS_short2.hmm
#- GlimmerHMM needs to be installed and executable by glimmerhmm_linux in the current directory
#- directory GlimmerHMM/trained_dir/ in working directory
#
#Files created:
#- coding-sequences (aminoacid): >name<_cds.aa
#- results of hmmsearch (whole genome): >name<_RM_hmm.res
#- results of hmmsearch (coding-sequences): >name<_cds_hmm.res
#- 1st set of predicted MADS-box-genes: >name<_cds_MADS.fa

#!/usr/bin/perl

use strict;
use warnings;
use LWP::Simple;
use Data::Dumper;

my %seqs2;
my %nucl_seqs;
my $nucl_sequence = "";
my $nucl_seqname = "";
my $cfile = "$ARGV[0]_cds.fa";
my $sfile = "$ARGV[0]_RM.fa";
my $file = "$ARGV[0]_cds.aa";
my $aminoacid_sequence = "";
my @est;
my @sco;
my $x = "";
my $max = 0;

sub nucl{
open(FILE, $cfile);						#imports cds.fa into a hash / key = name of a nucleotide-sequence
								#                           / value = nucleotide-sequence		
while(<FILE>){
  if ($_ =~/^>(.+)\s+$/){
     $nucl_seqs{$nucl_seqname} = $nucl_sequence;
     $nucl_seqname = $1;
     $nucl_seqname =~ s/\s//g;
     $nucl_sequence = "";
     }
  else {
     $_ =~ s/\n//g;
     $nucl_sequence = $nucl_sequence . $_;
     }
                            }
	$nucl_seqs{$nucl_seqname} = $nucl_sequence;
	   close(FILE);

open(FILE2, $sfile);						#imports RM.fa into a hash / key = name of a nucleotide-sequence
								#                           / value = nucleotide-sequence
while(<FILE2>){
  if ($_ =~/^>(.+)\s+$/){
     $seqs2{$nucl_seqname} = $nucl_sequence;
     $nucl_seqname = $1;
     $nucl_seqname =~ s/\s//g;
     $nucl_sequence = "";
     }
  else {
     $_ =~ s/\n//g;
     $nucl_sequence = $nucl_sequence . $_;
     }
                            }
	$seqs2{$nucl_seqname} = $nucl_sequence;
	   close(FILE2);
	   
      }

      

	my %code = ('AAA', 'K', 'AAC', 'N', 'AAG', 'K', 'AAT', 'N',                        #hash containing the code needed for translation of 
	            'ACA', 'T', 'ACC', 'T', 'ACG', 'T', 'ACT', 'T',		   	   #nucleotide-sequences into aminoacid-sequences
	            'AGA', 'R', 'AGC', 'S', 'AGG', 'R', 'AGT', 'S',
	            'ATA', 'I', 'ATC', 'I', 'ATG', 'M', 'ATT', 'I',
	            'CAA', 'Q', 'CAC', 'H', 'CAG', 'Q', 'CAT', 'H',
	            'CCA', 'P', 'CCC', 'P', 'CCG', 'P', 'CCT', 'P',
	            'CGA', 'R', 'CGC', 'R', 'CGG', 'R', 'CGT', 'R',
	            'CTA', 'L', 'CTC', 'L', 'CTG', 'L', 'CTT', 'L',
	            'GAA', 'E', 'GAC', 'D', 'GAG', 'E', 'GAT', 'D',
	            'GCA', 'A', 'GCC', 'A', 'GCG', 'A', 'GCT', 'A',
	            'GGA', 'G', 'GGC', 'G', 'GGG', 'G', 'GGT', 'G',
	            'GTA', 'V', 'GTC', 'V', 'GTG', 'V', 'GTT', 'V',
	            'TAA', '*', 'TAC', 'Y', 'TAG', '*', 'TAT', 'Y',
	            'TCA', 'S', 'TCC', 'S', 'TCG', 'S', 'TCT', 'S',
	            'TGA', '*', 'TGC', 'C', 'TGG', 'W', 'TGT', 'C',
	            'TTA', 'L', 'TTC', 'F', 'TTG', 'L', 'TTT', 'F');

		     
sub transl{
	print "Tranlasting cds in aminoacid sequences.\n";
        open (OUT, ">$file");									#translates nucleotide-sequences into aminoacid-sequences in 1st reading-frame
	  foreach my $name (sort keys %nucl_seqs){
	      my $nucleotide_sequence = $nucl_seqs{$name};
	  my $aminoacid_sequence = "";
	  for(my $i = 0; $i < length($nucleotide_sequence) - 3; $i += 3){
		      if (uc(substr($nucleotide_sequence, $i, 3)) =~ /[^ACGT]/){
			      $aminoacid_sequence .= "X";
		      }
		      else {
				$aminoacid_sequence .= $code{uc(substr($nucleotide_sequence, $i, 3))};
                      }}
	   if($name ne ""){									#avoids "empty" sequenzes
	    print OUT ">$name\n";	      
	    print OUT "$aminoacid_sequence\n";
	   }
		      }
	close(OUT);	      
		      }
		    

sub search1{
	print "Conducting HMMsearch for cds.\n";
 system ("rm $ARGV[0]_cds_hmm.res");
 system ("mkdir $ARGV[0]_trash");
 my $verzeichnis = "$ARGV[0]_trash";
 chdir ("$verzeichnis");
 system ("csplit -z ../$ARGV[0]_cds.aa '/>/' '{*}'");					#splits file containing all aminoacid-sequences into seperate files
 chdir ("..");
 opendir(DIR,$verzeichnis);								#=> sequences can be searched one by one
 while (my $fi = readdir(DIR)){
   if ($fi =~ /xx[0-9]+/){
    my $command = "hmmsearch MADS_short2.hmm $verzeichnis/$fi >>$ARGV[0]_cds_hmm.res";
    system("$command");	#searches each aminoacid-sequence by HMM-search
    }
  }
 closedir DIR;
}


sub main {
	my $hfile;
	my $ofile;
	my $tfile;
	my $pfile;
	my $blast;
	my $seqname = "";
	my $sequence = "";
	my %pseqs;
	my $readingframe;
	my $read;
	my $cds;
	my $id;

	$hfile = "$ARGV[0]_cds_hmm.res";
	$blast = "$ARGV[0].blast";
	$ofile = "$ARGV[0]_cds_MADS.fa";
	$tfile = "$ARGV[0]_RM.fa";
	$pfile = "$ARGV[0]_RM.aa";
	

		$seqname = "";
		$sequence = "";
		$readingframe = "";

		print "Reading translated genome\n";

		open(FILE, $pfile);
		while(<FILE>){								#imports RM.aa into a hash / key = name of an aminoacid-sequence
			if ($_ =~/^>(\d)_(.+)$/){					#		           / value = aminoacid-sequence
				$pseqs{$seqname}{$readingframe} = $sequence;
				$readingframe = $1;
				$seqname = $2;
				$seqname =~ s/\s//g;
				$sequence = "";
			}
			else {
				$_ =~ s/\n//g;
				$sequence = $sequence . $_;
			}
		}
		$pseqs{$seqname}{$readingframe} = $sequence;
		close(FILE);
		
		print "Reading hmmfile\n";
		
		open(OFILE, ">$ofile");						#writes 1st set of predicted MADS-box genes into a new file
		open(HMM, $hfile);
		$read = 4;
	while(<HMM>){
			$read++;
			if ($_ =~ />>\s+(.+)\s+$/){
				$cds = $1;
				$cds =~ s/\s//g;
			}
			elsif ($_ =~ /==\sdomain/){
				$read = 0;
			}
			elsif (($_ =~ /^\s+\w+\s+\d+\s+(\w+)/) && ($read == 3)){          #only writes a sequence into the file if presence of 
				if (checkDomProt($1)){					
					print OFILE ">$cds\n$nucl_seqs{$cds}\n";
				}
			}
		}
		close(HMM);
		close(OFILE);

                system("formatdb -i $tfile -pF");  
		system("blastall -v5 -b1 -p blastn -d $tfile -i $ofile >$blast");         #blasts 1st set of MADS-sequ + whole-genome sequences

		my $query;
		my $fragmentname;
		my $fragmentlength;
		my $fitlength;
		my $mask = 0;
		my $direction;
		my $first = 1;
		my $start;
		my $end;
		my $protstart;
		my $protend;

		print "BLASTing identified cds to genome to prevent double identification of MADS-box genes\n";
		
		open(BLAST, $blast);
		while(<BLAST>){
			if ($_ =~ /Query=\s+(.+)/){
				$query = $1;
			}
			elsif ($_ =~ />(.+)/){
				$fragmentname = $1;
			}
			elsif ($_ =~ /^\s+Length\s=\s(\d+)/){
				$fragmentlength = $1;
			}
			elsif ($_ =~ /^\s+Identities\s=\s\d+\/(\d+)\s\((\d+)%\)/){
				$fitlength = $1;
				$id = $2;
				if (($id > 97) && ($fitlength > 100)){
					$mask = 1;
				}
			}
			elsif (($_ =~ /^\s+Strand\s=\s\w+\s\/\s(\w+)/) && ($mask == 1)){
				$direction = $1;
				$first = 0;
			}
			elsif (($_ =~ /Sbjct:\s+(\d+)\s+\w+\s+(\d+)/) && ($mask == 1)){
				if ($first < 1){
					$start = $1;
				}
				$end = $2;
				$first++;
			}
			elsif ((($_ =~ /^\s+Score\s+=/) && ($mask == 1)) || (($_ =~ /BLASTN/) && ($mask == 1)) || (($_ =~ /Matrix/) && ($mask == 1))){
				if ($direction eq "Minus"){
					$start = $fragmentlength - $start;
					$end = $fragmentlength - $end;
				}
				$protstart = int($start / 3);
				$protend = int($end / 3);
				if ($direction eq "Minus"){
					for (my $k = $protstart; $k <= $protend; $k++){
						substr($pseqs{$fragmentname}{4},$k,1) = "X";
						substr($pseqs{$fragmentname}{5},$k,1) = "X";
						substr($pseqs{$fragmentname}{6},$k,1) = "X";
					}
				}
				else {
					for (my $k = $protstart; $k <= $protend; $k++){
						substr($pseqs{$fragmentname}{1},$k,1) = "X";
						substr($pseqs{$fragmentname}{2},$k,1) = "X";
						substr($pseqs{$fragmentname}{3},$k,1) = "X";
					}
				}
				$mask = 0;
			}

		}
		close(BLAST);

		my $outfile = $pfile . "_masked";						#creates a new file containing masked genomic sequences 
		open(FILE, ">$outfile");
		foreach my $key1 (sort keys %pseqs){
			foreach my $key2 (sort keys %{$pseqs{$key1}}){
				print FILE ">$key2" . "_$key1\n$pseqs{$key1}{$key2}\n";
			}
		}
		close(FILE);

}

sub checkDomProt { 
	my ($sequence) = @_;									
	my $urlstring = "http://www.ncbi.nlm.nih.gov/Structure/bwrpsb/bwrpsb.cgi?queries=" . "$sequence\n\n" . "&useid1=true&tdata=hits";
	my $mads_present = 0;
	my $content = get $urlstring;
	while (!defined $content){
		$content = get $urlstring;
	}
	my @lines = split(/\n+/, $content);
		
	foreach my $line (@lines){
		if ($line =~ /#cdsid\s+(.+)$/){
			my $cdsid = $1;
			$urlstring = "http://www.ncbi.nlm.nih.gov/Structure/bwrpsb/bwrpsb.cgi?cdsid=" . $cdsid . "&tdata=aligns&alnfmt=xml&dmode=all";
			$content = get $urlstring;
			my @llines = split(/\n+/, $content);
			my $done = 0;
			while ($done == 0){
				$done = 1;
				foreach my $lline (@llines){
					if ($lline =~ /Job\sis\sstill\srunning/){
						$done = 0;
					}
				}
				$content = get $urlstring;
				@llines = split(/\n+/, $content);
			}
			foreach my $lline (@llines){
				if ($lline =~ /^Q#1.+MADS/){
					$mads_present = 1;
				}
			}

		}
	}
	return($mads_present);
}

sub search2{
	print "Conducting HMMsearch for genome\n";
 system ("rm $ARGV[0]_RM_hmm.res");
 system ("mkdir $ARGV[0]_trash2");
 my $verzeichnis =  "$ARGV[0]_trash2";
 chdir ("$verzeichnis");
 system ("csplit -f yy -z ../$ARGV[0]_RM.aa_masked '/>/' '{*}'");					#splits file containing all aminoacid-sequences into seperate files
 chdir ("..");
 opendir(DIR,$verzeichnis);										#=> sequences can be searched one by one
 while (my $fi = readdir(DIR)){
   if ($fi =~ /yy[0-9]+/){
    system("hmmsearch MADS_short2.hmm $verzeichnis/$fi >>$ARGV[0]_RM_hmm.res");	#searches each aminoacid-sequence by HMM-search
   }
  }
  closedir DIR;
}


sub callGlimmer{
	
	my($counter, $species, $seq, $out_file, $name, $information) = @_;

	print "Call Glimmer\n";
		
	my $gene = "";
	my $line;
	my @coords;
		
	system("glimmerhmm_linux MADS/MADS" . $counter . ".txt GlimmerHMM/trained_dir/" . $species . "/ >glimmerHMM.out");
	
	open(GLIMMER, "glimmerHMM.out");

	while(<GLIMMER>) {
		$line = $_;
		if ($line =~/1/){
			@coords = split(/\s+/, $line);
			if ($coords[1] eq "1"){
				$gene .= substr($seq, $coords[5] - 1, $coords[6] - $coords[5] + 1);
			}
		}
	}
	if ($gene ne ""){
		open(OUT, ">>$out_file");
		print OUT (">MADS" . $counter . "_" . $name . "_" . $species . "\n");
		print OUT ($gene . "\n");
		close(OUT);
	}
	else {
		print("NO GENE FOUND FOR " . $information . " FOR TRAINING SET " . $species . "\n");
	}
}



sub predictORF {
	
	my($name, $frame, $start, $outfile, $counter) = @_;	
	print "Predict ORF\n";

	my $seq = "";
	my $revcomp = "";
	my $nuc = "";
	my $info = "$name" . "_$frame" . "_$start";

	my $substrstart;
	my $substrend;
		
	if ($frame > 3){
		# reverse complement of sequence
		$seq = $seqs2{$name};

		if ((length($seq) - (3 * $start) - 19700) >= 0){
			$substrstart = length($seq) - (3 * $start) - 19700;
		}
		else {
			$substrstart = 0;
		}
			
		if ((length($seq) - (3 * $start) + 300) < length($seq)){
				$substrend = length($seq) - (3 * $start) + 300;
		}
		else {
			$substrend = length($seq) - 1;
		}

		$seq = substr($seq, $substrstart, $substrend - $substrstart);

		$revcomp = "";
		for (my $j = length($seq); $j > 0; $j--){
			$nuc = substr($seq, $j-1, 1);
			if (($nuc eq "N") || ($nuc eq "n")){
				$revcomp .= "N";
			}
			elsif (($nuc eq "A") || ($nuc eq "a")){
				$revcomp .= "T";
			}
			elsif (($nuc eq "C") || ($nuc eq "c")){
				$revcomp .= "G";
			}
			elsif (($nuc eq "G") || ($nuc eq "g")){
				$revcomp .= "C";
			}
			elsif (($nuc eq "T") || ($nuc eq "t")){
					$revcomp .= "A";
			}
			else {
				print "SOMETHING ELSE IN SEQUENCE: " . $nuc . "\n";
				$revcomp .= "n";
			}
		}
		
		chdir("MADS");
		my $fn = ">MADS$counter.txt";
		open(OUT, $fn);
		print OUT (">seq\n$revcomp\n");
		close(OUT);
		chdir("..");

		callGlimmer($counter, "arabidopsis", $revcomp, $outfile, $name, $info);
		callGlimmer($counter, "rice", $revcomp, $outfile, $name, $info);
			
	}
	else {
		$seq = $seqs2{$name};

		if (((3 * $start) - 300) >= 0){
			$substrstart = (3 * $start) - 300;
		}
		else {
			$substrstart = 0;
		}
	
		if (((3 * $start) + 19700) < length($seq)){
			$substrend = (3 * $start) + 19700;
		}
		else {
			$substrend = length($seq) - 1;
		}
				
		$seq = substr($seq, $substrstart, $substrend - $substrstart);
		chdir("MADS");
		my $fn = ">MADS$counter.txt";
		open(OUT, $fn);
		print OUT ">seq\n$seq\n";
		close(OUT);
		chdir("..");
		
		callGlimmer($counter, "arabidopsis", $seq, $outfile, $name, $info);
		callGlimmer($counter, "rice", $seq, $outfile, $name, $info);

	}
}


sub write{
  my $ofile2 = "$ARGV[0]_RM_MADS.fa";
  my $hfile2 = "$ARGV[0]_RM_hmm.res";
  my $read;
  my $cds;
  my $rm;
  my $frame = 0;
  my $start = 0;
  my $counter = 0;
  system("mkdir MADS");

		open(HMM2, $hfile2);
		$read = 4;
		while(<HMM2>){
			$read++;
			if ($_ =~ />>\s+(.+)\s+$/){
				$cds = $1;
				$cds =~ s/\s//g;
				if ($cds =~ /^(\d)_/){
					$frame = $1;
				}
				$rm = $cds;
				$rm =~ s/[0-9]*_//;
			}
			elsif ($_ =~ /==\sdomain/){
				$read = 0;
			}
			elsif (($_ =~ /^\s+\w+\s+(\d+)\s+(\w+)/) && ($read == 3)){
				$start = $1;
				if (checkDomProt($2)){					#only writes a sequence into the file if presence of
					$counter++;
					predictORF($rm, $frame, $start, $ofile2, $counter);
				}
			}
                  }			
}



if($ARGV[0] eq ""){
 print "\ncall programm like this: perl MADSprediction_pipeline.pl >name common to the given sequence-files< (insert 'help' for help)\n";
 }
elsif($ARGV[0] eq "help"){
 print "files needed:\nwhole-genome-sequences (nucleotide): >name<_RM.fa\nwhole-genome-sequences (aminoacid): >name<_RM.aa\ncoding-sequences (nucleotide): >name<_cds.fa\n\nfiles created:\ncoding-sequences (aminoacid): >name<_cds.aa\nresults of hmmsearch (whole genome): >name<_RM_hmm.res\nresults of hmmsearch (coding-sequences): >name<_cds_hmm.res\n1st set of predicted MADS-box-genes: >name<_cds_MADS.fa\nfiles needed for blast search: >name<_RM.fa.nhr/.nin/.nsq\nblast-results: >name<.blast\nmasked genomic sequences: >name<_RM.aa_masked\n2nd set of predicted MADS-box-genes: >name<_cds_MADS.fa\nresults of est-search (inside new directory '>name<'): *_MADS_res.fa\n"
}
else{
 &nucl;
 &transl;
 &search1;
 &main;
 &search2;
 &write;
}


